local MinBuild, MaxBuild, Build = 40000, 0, select(4, GetBuildInfo())
if Build < MinBuild or ( MaxBuild > 0 and Build > MaxBuild ) then return end
local s = SpellFlashAddon
local x = s.UpdatedVariables

BittensSpellFlashLibrary2 = {}
local c = BittensSpellFlashLibrary2
local AddonName
local a

local libraryAddonName, libraryTable = ...
BINDING_HEADER_BITTENS_SPELLFLASH = select(2, GetAddOnInfo(libraryAddonName))
BINDING_NAME_BITTENS_SPELLFLASH_DEBUGGING
    = libraryTable.Localize["Print Debugging Info"]

local HideHilighting = false
hooksecurefunc(
    "ActionButton_ShowOverlayGlow", 
    function(self)
        if HideHilighting and self.overlay then
            self.overlay:Hide()
            ActionButton_HideOverlayGlow(self)
        end
    end)
s.AddSettingsListener(
    function()
        HideHilighting = false
    end)

c.Debugging = false

c.SLOW_MELEE_DEBUFFS = {
    68055, -- Judgements of the Just 
    58180, -- Infected Wounds 
    59921, -- Frost Fever 
    6343, -- Thunder Clap 
    8042, -- Earth Shock 
    50285, -- Dust Cloud 
    16914, -- Hurricane 
    90314, --Tailspin
}

c.ARMOR_DEBUFFS = {
    7386, -- Sunder Armor
    8647, -- Expose Armor
    91565, -- Faerie Fire
    35387, -- Corrosive Spit
    50498, -- Tear Armor
}

c.PHYSICAL_DAMAGE_DEBUFFS = {
    702, -- Curse of Weakness
    99, -- Demoralizing Roar
    1160, -- Demoralizing Shout 
    81130, -- Scarlet Fever 
    26017, -- Vindication
}

c.BLEED_DEBUFFS = {
    33876, -- Mangle
    46857, -- Trauma
    16511, -- Hemorrhage
    50271, -- Tendon Rip
    35290, -- Gore
    57386, -- Stampede
}

c.FIVE_PERCENT_BUFFS = {
    1126, -- Mark of the Wild
    20217, -- Blessing of Kings
    90363, -- Embrace of the Shale Spider
}

c.STAMINA_BUFFS = {
    79105, -- Power Word: Fortitude
    90364, -- Qiraji Fortitude
    6307, -- Blood Pact
    469, -- Commanding Shout
}

c.SHADOW_RESISTANCE_BUFFS = {
    19891, -- Resistance Aura
    79107, -- Shadow Protection
}

c.INTELLECT_BUFFS = {
    1459, -- Arcane Brilliance
    61316, -- Dalaran Brilliance
    52109, -- Flametongue Totem
    54424, -- Fel Intelligence
}

c.MANA_BUFFS = {
    1459, -- Arcane Brilliance
    61316, -- Dalaran Brilliance
    54424, -- Fel Intelligence
}

c.ATTACK_POWER_BUFFS = {
    79102, -- Blessing of Might
    53138, -- Abomination's Might
    19506, -- Trueshot Aura
    30808, -- Unleashed Rage
}

c.MP5_BUFFS = {
    19740, -- Blessing of Might
    5675, -- Mana Spring Totem
    54424, -- Fel Intelligence
}

function c.Init(addonName, addonTable)
    AddonName = addonName
    a = addonTable
end

function c.Run(...)
    local ranOne = false
    for i=1,select("#", ...) do
        local runInfo = select(i, ...)
        
        local func = runInfo[1]
        local spec = runInfo[2]
        local offSwitch = runInfo[3]
        local form = runInfo[4]
        
        if s.TalentMastery(spec) 
            and not a.GetConfig(offSwitch) 
            and (form == nil or s.Form(form)) then
            
            func()
            ranOne = true
        end
    end
    HideHilighting = ranOne
end

-- remove in favor of a.GetConfig() when possible
function c.Selected(switch)
    return s.GetModuleConfig(AddonName, switch)
end

function c.RotateCooldowns(uncontrolledBuffs, ...)
    for tmp,id in pairs(uncontrolledBuffs) do
        if s.Buff(id, "player") then
            return
        end
    end
    for i=1,select("#", ...) do
        local buff = select(i, ...)
        local spell = a.spells[buff]
        if spell and spell.Buff then
            buff = spell.Buff
        end
        if c.HasBuff(buff, spell.NoGCD, spell.EarlyRefresh) then
            return
        end
    end
    return a.Flash(...)
end

function c.HasBuff(name, noGCD, durationRequired)
    local duration = s.BuffDuration(c.GetID(name), "player")
    
    -- duration == 0 for permanent buffs, so check that here   
    if duration == 0 then
        return s.Buff(c.GetID(name), "player")
    end
    
    return math.max(duration - c.GetBusyTime(noGCD), 0)
end

function c.GetBuffDuration(name, noGCD)
    return math.max(
        s.BuffDuration(c.GetID(name), "player") - c.GetBusyTime(noGCD), 0)
end

function c.GetBuffStack(name)
    if c.HasBuff(name) then
        return s.BuffStack(c.GetID(name), "player")
    else
        return 0
    end
end

function c.HasMyDebuff(name, noGCD, durationRequired)
    if not durationRequired then
        durationRequired = .2
    end
    return c.GetMyDebuffDuration(name, noGCD) > durationRequired
end

function c.GetMyDebuffDuration(name, noGCD)
    return math.max(s.MyDebuffDuration(c.GetID(name)) - c.GetBusyTime(noGCD), 0)
end

function c.GetDebuffDuration(name, noGCD)
    return math.max(s.DebuffDuration(c.GetID(name)) - c.GetBusyTime(noGCD), 0)
end

function c.GetBusyTime(noGCD)
    local casting = s.GetCastingOrChanneling(nil, "player")
    if noGCD then
        return casting
    else
        return math.max(casting, s.GlobalCooldown())
    end
end

function c.GetCooldown(name)
    return math.max(s.SpellCooldown(c.GetID(name)) - c.GetBusyTime(), 0)
end

function c.GetIDs(...)
    local ids = {}
    for i=1,select("#", ...) do
        local id = c.GetID(select(i, ...))
        if id then
            table.insert(ids, id)
        end
    end
    return ids
end

function c.GetID(name)
    local spell = a.spells[name]
    if spell and spell.ID then
        return spell.ID
    else
        print("No spell defined (or no ID attribute):", name)
    end
end

function c.SelfBuffNeeded(name)
    if s.InCombat() then
        return not c.HasBuff(name)
    else
        return not c.HasBuff(name, false, 5 * 60)
    end
end

function c.RaidBuffNeeded(idTable)
    local duration = 0
    if not s.InCombat() then
        duration = 5 * 60
    end
    local flags = "raid|all|range"
    if idTable == c.INTELLECT_BUFFS then
        flags = flags .. "|mana"
    end
    return not s.Buff(idTable, flags, duration)
end

function c.Debug(tag, ...)
    if c.Debugging then
        return print("|cFF00FFFF["..tag.."]|r", ...)
    end
end

function c.IsCasting(name)
    local id = c.GetID(name)
    return s.CurrentSpell(id) or s.Casting(id, "player")
end

function c.IsCastingOrInAir(name)
    return c.IsCasting(name) or s.SpellDelay(c.GetID(name))
end

function c.CheckFirstForTaunts(z)
    local primaryTarget = s.GetPrimaryThreatTarget()
    if not primaryTarget or UnitIsUnit(primaryTarget, "player") then
        return false
    end
    
    if UnitGroupRolesAssigned(primaryTarget) == "TANK" then
        z.FlashSize = s.FlashSizePercent() / 3
        z.FlashColor = "yellow"
    else
        z.FlashSize = nil
        z.FlashColor = "red"
    end
    return true
end

function c.IsInGroup()
    return GetNumPartyMembers() > 0 or GetNumRaidMembers() > 1
end

function c.SpellsAreEqual(blizzardSpellName, codeSpellName)
    return blizzardSpellName == s.SpellName(c.GetID(codeSpellName), 1)
end
